@@ -2,6 +2,7 @@
 # See LICENSE file for full licensing details.
 import json
 from collections import OrderedDict
+from datetime import datetime
 
 from formiodata.utils import base64_encode_url, decode_resource_template, fetch_dict_get_value
 
@@ -60,12 +61,20 @@ class Component:
     def value(self, value):
         self.form['value'] = value
 
+    @property
+    def raw_value(self):
+        return self.form['raw_value']
+
+    @raw_value.setter
+    def raw_value(self, value):
+        self.form['raw_value'] = value
+
     @property
     def hidden(self):
         return self.raw.get('hidden')
 
     def render(self):
-        self.html_component = f"<p>{self.form.get('value')}</p>"
+        self.html_component = '<p>%s</p>' % self.form.get('value')
 
 
 # Basic
@@ -193,11 +202,113 @@ class phoneNumberComponent(Component):
 
 
 class datetimeComponent(Component):
-    pass
 
+    def _format_mappings(self):
+        """
+        Dictionary of mappings between Formio Datetime component
+        (key) to Python format (value).
 
-class dateComponent(Component):
-    pass
+        Formio uses the format codes refererend in:
+        https://github.com/angular-ui/bootstrap/tree/master/src/dateparser/docs#uibdateparsers-format-codes
+        """
+        return {
+            'year': {'yyyy': '%Y', 'yy': '%y', 'y': '%y'},
+            'month': {'MMMM': '%B', 'MMM': '%b', 'MM': '%m', 'M': '%-m'},
+            'day': {'dd': '%d', 'd': '%-d'},
+            'hour': {'HH': '%HH', 'H': '%-H', 'hh': '%I', 'h': '%-I'},
+            'minute': {'mm': '%M', 'm': '%-M'},
+            'second': {'ss': '%S', 's': '%-S'},
+            'am_pm': {'a': '%p'}
+        }
+
+    def _fromisoformat(self, value):
+        # Backport of Python 3.7 datetime.fromisoformat
+        if hasattr(datetime, 'fromisoformat'):
+            # == Python 3.7
+            return datetime.fromisoformat(value)
+        else:
+            # < Python 3.7
+            fmt_str =  r"%Y-%m-%dT%H:%M:%S.%f" # replaces the fromisoformatm, not available in python 3.6
+            return datetime.datetime.strptime(value, fmt_str)
+
+    @property
+    def value(self):
+        return super().value
+
+    @value.setter
+    def value(self, value):
+        """ Inherit property setter the right way, URLs:
+        - https://gist.github.com/Susensio/979259559e2bebcd0273f1a95d7c1e79
+        - https://stackoverflow.com/questions/35290540/understanding-property-decorator-and-inheritance
+        """
+        # TODO: to improve these transformations (mappings and loops)
+
+        if not value:
+            return value
+
+        component = self.builder.components.get(self.key)
+        dt = self._fromisoformat(value)
+        py_dt_format = formio_dt_format = component.raw.get('format')
+        mapping = self._format_mappings()
+
+        # year
+        done = False
+        for formio, py in mapping['year'].items():
+            if not done and formio in formio_dt_format:
+                py_dt_format = py_dt_format.replace(formio, py)
+                done = True
+
+        # month
+        done = False
+        for formio, py in mapping['month'].items():
+            if not done and formio in formio_dt_format:
+                py_dt_format = py_dt_format.replace(formio, py)
+                done = True
+
+        #day
+        done = False
+        for formio, py in mapping['day'].items():
+            if not done and formio in formio_dt_format:
+                py_dt_format = py_dt_format.replace(formio, py)
+                done = True
+
+        # hour
+        done = False
+        for formio, py in mapping['hour'].items():
+            if not done and formio in formio_dt_format:
+                py_dt_format = py_dt_format.replace(formio, py)
+                done = True
+
+        # minute
+        done = False
+        for formio, py in mapping['minute'].items():
+            if not done and formio in formio_dt_format:
+                py_dt_format = py_dt_format.replace(formio, py)
+                done = True
+
+        # second
+        done = False
+        for formio, py in mapping['second'].items():
+            if not done and formio in formio_dt_format:
+                py_dt_format = py_dt_format.replace(formio, py)
+                done = True
+
+        # 12 hours AM/PM
+        done = False
+        for formio, py in mapping['am_pm'].items():
+            if not done and formio in formio_dt_format:
+                py_dt_format = py_dt_format.replace(formio, py)
+                done = True
+
+        val = dt.strftime(py_dt_format)
+        super(self.__class__, self.__class__).value.fset(self, val)
+
+    def to_datetime(self):
+        if not self.raw_value:
+            return None
+        component = self.builder.components.get(self.key)
+        dt = self._fromisoformat(self.raw_value)
+        return dt
 
 
 class timeComponent(Component):
@@ -386,4 +497,4 @@ class resourceComponent(Component):
 
     @property
     def values(self):
-        return self.raw.get('data').get('values')
\ No newline at end of file
+        return self.raw.get('data').get('values')
